from __future__ import print_function
from django.core.management.base import BaseCommand
import os
import subprocess
from instant.conf import CENTRIFUGO_PORT


class Command(BaseCommand):
    help = 'Run the Centrifugo websockets server'

    def handle(self, *args, **options):
        basepath = os.getcwd()
        c = basepath + "/centrifugo/centrifugo"
        conf = basepath + "/centrifugo/config.json"
        cmd = [
            c,
            "--config", conf,
            "--port", str(CENTRIFUGO_PORT)
        ]
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        for line in p.stdout:
            msg = str(line).replace("b'", "")
            msg = msg[0:-3]
            print(msg)
        p.wait()
        return



# -*- coding: utf-8 -*-

from django import forms
from instant.conf import ENABLE_USERS_CHANNEL, ENABLE_STAFF_CHANNEL, ENABLE_SUPERUSER_CHANNEL, PUBLIC_CHANNEL, USERS_CHANNELS, STAFF_CHANNELS, SUPERUSER_CHANNELS


choices = [((PUBLIC_CHANNEL, 'Public'))]
if ENABLE_USERS_CHANNEL is True:
    for channel in USERS_CHANNELS:
        choices.append((channel, channel))
if ENABLE_STAFF_CHANNEL is True:
    for channel in STAFF_CHANNELS:
        choices.append((channel, channel))
if ENABLE_SUPERUSER_CHANNEL is True:
    for channel in SUPERUSER_CHANNELS:
        choices.append((channel, channel))


class BroadcastForm(forms.Form):
    message = forms.CharField(max_length=300, label="Message", widget=forms.Textarea(
        attrs={'rows': '2'}), required=True)
    event_class = forms.CharField(
        max_length=60, label="Event class", required=False)
    default_channel = forms.CharField(
        max_length=60, label="Channels", required=False, widget=forms.RadioSelect(choices=choices))
    channel = forms.CharField(
        max_length=60, label="Other channel", required=False)



# -*- coding: utf-8 -*-
import time
from cent.core import generate_token
from django import template
from django.template.defaultfilters import stringfilter
from django.conf import settings
from django.utils.html import mark_safe
from django.core.exceptions import ImproperlyConfigured
from ..apps import CHANNELS, HANDLERS

DEBUG = False

try:
    SECRET_KEY = getattr(settings, 'CENTRIFUGO_SECRET_KEY')
except ImportError:
    raise ImproperlyConfigured(
        u"The Centrifugo secret key must be set in settings.py with CENTRIFUGO_SECRET_KEY")
SITE_SLUG = getattr(settings, 'SITE_SLUG', 'site')
CENTRIFUGO_HOST = getattr(settings, 'CENTRIFUGO_HOST', 'http://localhost')
CENTRIFUGO_PORT = getattr(settings, 'CENTRIFUGO_PORT', 8001)

APPS = getattr(settings, 'INSTANT_APPS', [])

public_channel = SITE_SLUG + '_public'
PUBLIC_CHANNEL = getattr(settings, 'INSTANT_PUBLIC_CHANNEL', public_channel)
ENABLE_PUBLIC_CHANNEL = getattr(
    settings, 'INSTANT_ENABLE_PUBLIC_CHANNEL', True)
ENABLE_STAFF_CHANNEL = getattr(settings, 'INSTANT_ENABLE_STAFF_CHANNEL', False)
ENABLE_USERS_CHANNEL = getattr(settings, 'INSTANT_ENABLE_USERS_CHANNEL', False)
ENABLE_SUPERUSER_CHANNEL = getattr(
    settings, 'INSTANT_ENABLE_SUPERUSER_CHANNEL', False)
EXCLUDE = getattr(settings, 'INSTANT_EXCLUDE', ["__presence__"])

PUBLIC_CHANNELS = getattr(settings, 'INSTANT_PUBLIC_CHANNELS', ())
USERS_CHANNELS = getattr(
    settings, 'INSTANT_USERS_CHANNELS', ())
STAFF_CHANNELS = getattr(settings, 'INSTANT_STAFF_CHANNELS', ())
SUPERUSER_CHANNELS = getattr(settings, 'INSTANT_SUPERUSER_CHANNELS', ())
USERS_CHANNELS = getattr(settings, 'INSTANT_USERS_CHANNELS', ())

# javascript debug messages
debug_mode = getattr(settings, 'INSTANT_DEBUG', False)
if debug_mode is True:
    DEBUG_MODE = "true"
else:
    DEBUG_MODE = "false"

register = template.Library()


@register.simple_tag
def get_centrifugo_url():
    return CENTRIFUGO_HOST + ":" + str(CENTRIFUGO_PORT)


@register.simple_tag
def debug_mode():
    return DEBUG_MODE


@register.simple_tag
def get_timestamp():
    return str(int(time.time()))


@register.simple_tag
def mq_generate_token(user, timestamp, info=""):
    token = generate_token(SECRET_KEY, user, timestamp, info)
    return token


@register.simple_tag
def public_channel_is_on():
    return ENABLE_PUBLIC_CHANNEL


@register.simple_tag
def get_public_channel():
    return PUBLIC_CHANNEL


@register.simple_tag
def is_users_channel():
    return ENABLE_USERS_CHANNEL


@register.simple_tag
def get_users_channel():
    return '$' + SITE_SLUG + '_users'


@register.simple_tag
def is_staff_channel():
    return ENABLE_STAFF_CHANNEL


@register.simple_tag
def get_staff_channel():
    return '$' + SITE_SLUG + '_staff'


@register.simple_tag
def is_superuser_channel():
    return ENABLE_SUPERUSER_CHANNEL


@register.simple_tag
def get_superuser_channel():
    return '$' + SITE_SLUG + '_admin'


def _get_channels_for_role(path, role):
    if role == "all":
        role_chans = CHANNELS["public"] + CHANNELS["users"] + \
            CHANNELS["staff"] + CHANNELS["superuser"]
    else:
        role_chans = CHANNELS[role]
    chans = []
    if path.endswith("/"):
        path = path[:-1]
    for chan in role_chans:
        if chan["path"] is not None:
            for chanpath in chan["path"]:
                if chanpath.endswith("/"):
                    chanpath = chanpath[:-1]
                if chanpath == path:
                    chans.append(chan["slug"])
                    break
        else:
            chans.append(chan["slug"])
    return chans


@register.simple_tag
def get_all_channels():
    chans = []
    for ctype in CHANNELS:
        for chan in CHANNELS[ctype]:
            chans.append(chan["slug"])
    return chans


@register.simple_tag
def get_channels_for_role(path, role):
    chans = _get_channels_for_role(path, role)
    return chans


@register.simple_tag
def get_handlers_url(chan):
    if chan in HANDLERS:
        url = "instant/handlers/" + chan + ".js"
    else:
        url = "instant/handlers/default.js"
    name = chan.replace("$", "")
    return [url, name]


@register.simple_tag
def get_apps():
    return APPS


@register.filter
@stringfilter
def is_in_apps(app):
    if app in APPS:
        return True
    else:
        return False


@register.simple_tag
def get_default_channels():
    channels = []
    if ENABLE_PUBLIC_CHANNEL is True:
        channels.append(PUBLIC_CHANNEL)
    if ENABLE_STAFF_CHANNEL is True:
        c = '$' + SITE_SLUG + '_staff'
        channels.append(c)
    if ENABLE_USERS_CHANNEL is True:
        c = '$' + SITE_SLUG + '_users'
        channels.append(c)
    if ENABLE_SUPERUSER_CHANNEL is True:
        c = '$' + SITE_SLUG + '_admin'
        channels.append(c)
    return channels


@register.simple_tag
def exclude_chans():
    chans = []
    for chan in EXCLUDE:
        chans.append('"' + chan + '"')
    return mark_safe(",".join(chans))


@register.simple_tag
def num_excluded_chans():
    return len(EXCLUDE)



# -*- coding: utf-8 -*-
from cent.core import generate_channel_sign
from .conf import PUBLIC_CHANNEL, SECRET_KEY, PUBLIC_CHANNELS, \
    USERS_CHANNELS, STAFF_CHANNELS, SUPERUSER_CHANNELS, ENABLE_PUBLIC_CHANNEL, \
    ENABLE_USERS_CHANNEL, ENABLE_STAFF_CHANNEL, ENABLE_SUPERUSER_CHANNEL, \
    DEFAULT_USERS_CHANNEL, DEFAULT_STAFF_CHANNEL, DEFAULT_SUPERUSER_CHANNEL


def signed_response(channel, client):
    signature = generate_channel_sign(SECRET_KEY, client, channel, info="")
    return {"sign": signature}


def _get_public_channel():
    return PUBLIC_CHANNEL


def _ensure_channel_is_private(chan):
    if chan.startswith("$") is False:
        return "$" + chan
    return chan


def _check_chanconf(chanconf, private=True):
    chan = chanconf[0]
    if private is True:
        chan = _ensure_channel_is_private(chan)
    path = None
    if len(chanconf) > 1:
        path = chanconf[1]
    chan = dict(slug=chan, path=path)
    return chan


def channels_for_role(role):
    """
    Get the channels for a role
    """
    chans = []
    chans_names = []
    # default channels
    if role == "public":
        if ENABLE_PUBLIC_CHANNEL is True:
            chan = dict(slug=PUBLIC_CHANNEL, path=None)
            chans.append(chan)
            chans_names.append(PUBLIC_CHANNEL)
    elif role == "users":
        if ENABLE_USERS_CHANNEL is True:
            chan = _ensure_channel_is_private(DEFAULT_USERS_CHANNEL)
            chan = dict(slug=chan, path=None)
            chans.append(chan)
            chans_names.append(DEFAULT_USERS_CHANNEL)
    elif role == "staff":
        if ENABLE_STAFF_CHANNEL is True:
            chan = _ensure_channel_is_private(DEFAULT_STAFF_CHANNEL)
            chan = dict(slug=chan, path=None)
            chans.append(chan)
            chans_names.append(DEFAULT_STAFF_CHANNEL)
    elif role == "superuser":
        if ENABLE_SUPERUSER_CHANNEL is True:
            chan = _ensure_channel_is_private(DEFAULT_SUPERUSER_CHANNEL)
            chan = dict(slug=chan, path=None)
            chans.append(chan)
            chans_names.append(DEFAULT_SUPERUSER_CHANNEL)
    # declared channels
    if role == "public":
        if len(PUBLIC_CHANNELS) > 0:
            for chanconf in PUBLIC_CHANNELS:
                chan = _check_chanconf(chanconf, False)
                chans.append(chan)
                chans_names.append(chan["slug"])
    elif role == "users":
        if len(USERS_CHANNELS) > 0:
            for chanconf in USERS_CHANNELS:
                chan = _check_chanconf(chanconf)
                chans.append(chan)
                chans_names.append(chan["slug"])
    elif role == "staff":
        if len(STAFF_CHANNELS) > 0:
            for chanconf in STAFF_CHANNELS:
                chan = _check_chanconf(chanconf)
                chans.append(chan)
                chans_names.append(chan["slug"])
    elif role == "superuser":
        if len(SUPERUSER_CHANNELS) > 0:
            for chanconf in SUPERUSER_CHANNELS:
                chan = _check_chanconf(chanconf)
                chans.append(chan)
                chans_names.append(chan["slug"])
    return chans, chans_names


def get_channels_for_roles():
    chans = dict(public=[], users=[], staff=[], superuser=[])
    chans_names = chans.copy()
    roles = ["public", "users", "staff", "superuser"]
    for role in roles:
        ch, chn = channels_for_role(role)
        chans[role] = ch
        chans_names[role] = chn
    return chans, chans_names



from setuptools import setup, find_packages


version = __import__('instant').__version__

setup(
    name='django-instant',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    description='Websockets for Django with Centrifugo ',
    author='synw',
    author_email='synwe@yahoo.com',
    url='https://github.com/synw/django-instant',
    download_url='https://github.com/synw/django-instant/releases/tag/' + version,
    keywords=['django', 'websockets', 'centrifugo'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'cent',
    ],
    zip_safe=False
)



from __future__ import print_function
import os
import subprocess
import json
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Install the Centrifugo websockets server for Linux'

    def handle(self, *args, **options):
        centrifugo_version = "1.7.6"
        fetch_url = "https://github.com/centrifugal/centrifugo/releases/download/v" + \
            centrifugo_version + "/centrifugo-" + centrifugo_version + "-linux-386.zip"
        subprocess.call(["wget", fetch_url])
        dirname = "centrifugo-" + centrifugo_version + "-linux-386"
        subprocess.call(["unzip", dirname + ".zip"])
        subprocess.call(["mv", dirname, "centrifugo"])
        subprocess.call(["rm", "-f", dirname + ".zip"])
        subprocess.call(["centrifugo/centrifugo", "genconfig"])
        subprocess.call(["mv", "config.json", "centrifugo"])
        # generate settings
        print("Updating settings")
        basepath = os.getcwd()
        project_name = settings.BASE_DIR.split("/")[-1:][0]
        filepath = basepath + '/centrifugo/config.json'
        filepathtmp = basepath + '/tmp.py'
        with open(filepath) as f:
            content = f.read()
        f = open(filepath, "r")
        f2 = open(filepathtmp, "w+")
        conf = json.loads(content)
        conf["anonymous"] = "true"
        key = conf["secret"]
        res = json.dumps(conf, indent=4).replace('"true"', 'true')
        f2.write(res)
        f2.close()
        os.remove(filepath)
        os.rename(filepathtmp, filepath)
        # settings.py
        extralines = 'SITE_SLUG = "' + project_name + '"\n'
        extralines = extralines + 'SITE_NAME = SITE_SLUG\n'
        extralines = extralines + 'CENTRIFUGO_SECRET_KEY = "' + key + '"\n'
        extralines = extralines + 'CENTRIFUGO_HOST = "http://localhost"\n'
        extralines = extralines + "CENTRIFUGO_PORT = 8001\n"
        project_dir = basepath + "/" + project_name
        filepath = project_dir + '/settings.py'
        f = open(filepath, "a")
        f.write("\n" + extralines + "\n")
        f.close()
        print("The Centrifugo websockets server is installed. Run it with python3 manage.py runws")
        return



from django.conf import settings


SECRET_KEY = getattr(settings, 'CENTRIFUGO_SECRET_KEY', "secret_key")
CENTRIFUGO_HOST = getattr(settings, 'CENTRIFUGO_HOST', 'http://localhost')
CENTRIFUGO_PORT = getattr(settings, 'CENTRIFUGO_PORT', 8001)

SITE_SLUG = getattr(settings, 'SITE_SLUG', 'site')
SITE_NAME = getattr(settings, 'SITE_NAME', 'Site')

BROADCAST_WITH = getattr(settings, 'INSTANT_BROADCAST_WITH', "py")

public_channel = SITE_SLUG + '_public'
PUBLIC_CHANNEL = getattr(settings, 'INSTANT_PUBLIC_CHANNEL', public_channel)
ENABLE_PUBLIC_CHANNEL = getattr(
    settings, 'INSTANT_ENABLE_PUBLIC_CHANNEL', True)
ENABLE_USERS_CHANNEL = getattr(settings, 'INSTANT_ENABLE_USERS_CHANNEL', False)
ENABLE_STAFF_CHANNEL = getattr(settings, 'INSTANT_ENABLE_STAFF_CHANNEL', False)
ENABLE_SUPERUSER_CHANNEL = getattr(
    settings, 'INSTANT_ENABLE_SUPERUSER_CHANNEL', False)

DEFAULT_USERS_CHANNEL = "$" + SITE_SLUG + "_users"
DEFAULT_STAFF_CHANNEL = "$" + SITE_SLUG + "_staff"
DEFAULT_SUPERUSER_CHANNEL = "$" + SITE_SLUG + "_admin"

PUBLIC_CHANNELS = getattr(settings, 'INSTANT_PUBLIC_CHANNELS', [])
USERS_CHANNELS = getattr(
    settings, 'INSTANT_USERS_CHANNELS', [])
STAFF_CHANNELS = getattr(settings, 'INSTANT_STAFF_CHANNELS', [])
SUPERUSER_CHANNELS = getattr(settings, 'INSTANT_SUPERUSER_CHANNELS', [])
USERS_CHANNELS = getattr(settings, 'INSTANT_USERS_CHANNELS', [])


EXCLUDE = getattr(settings, 'INSTANT_EXCLUDE', ["__presence__"])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import json
from django.apps import AppConfig
from django.utils._os import safe_join


HANDLERS = []
CHANNELS = {}
CHANNELS_NAMES = {}


class InstantConfig(AppConfig):
    name = 'instant'

    def ready(self):
        global HANDLERS, CHANNELS, CHANNELS_NAMES
        from django.conf import settings
        from .utils import get_channels_for_roles
        from .conf import ENABLE_STAFF_CHANNEL, ENABLE_USERS_CHANNEL, \
            ENABLE_SUPERUSER_CHANNEL
        col = '\033[91m'
        endcol = '\033[0m'
        if ENABLE_STAFF_CHANNEL is True:
            print(col + "Warning from Django Instant" + endcol + ": the setting ENABLE_STAFF_CHANNEL "
                  "will be deprecated in version 0.8. "
                  "Please use declarative channels instead")
        if ENABLE_USERS_CHANNEL is True:
            print(col + "Warning from Django Instant" + endcol + ": the setting ENABLE_USERS_CHANNEL "
                  "will be deprecated in version 0.8. "
                  "Please use declarative channels instead")
        if ENABLE_SUPERUSER_CHANNEL is True:
            print(col + "Warning from Django Instant" + endcol + ": the setting ENABLE_SUPERUSER_CHANNEL "
                  "will be deprecated in version 0.8. "
                  "Please use declarative channels instead")

        CHANNELS, CHANNELS_NAMES = get_channels_for_roles()
        try:
            if settings.INSTANT_DEBUG is True:
                print("Django Instant registered channels:")
                print(json.dumps(CHANNELS, indent=2))
        except AttributeError:
            pass
        # check if the default handler exists
        try:
            d = "templates/instant/handlers"
            handlers_dir = safe_join(settings.BASE_DIR, d)
            # map default handlers
            handlers = os.listdir(handlers_dir)
            for handler in handlers:
                HANDLERS.append(handler.replace(".js", ""))
            try:
                if settings.INSTANT_DEBUG is True:
                    print("Handlers:", HANDLERS)
            except AttributeError:
                pass
        except FileNotFoundError as e:
            try:
                if settings.INSTANT_DEBUG is True:
                    print("No handlers found for custom channels")
            except AttributeError:
                pass
        except Exception as e:
            raise(e)
